package dev.shadowsoffire.apotheosis.adventure.affix;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Multimap;
import dev.shadowsoffire.apotheosis.Apoth;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.AdventureModule;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixType;
import dev.shadowsoffire.apotheosis.adventure.affix.AttributeAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.CatalyzingAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.CleavingAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.DamageReductionAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.DurableAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.EnlightenedAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.ExecutingAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.FestiveAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.MagicalArrowAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.OmneticAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.PotionAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.PsychicAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.RadialAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.RetreatingAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.SpectralShotAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.TelepathicAffix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.ThunderstruckAffix;
import dev.shadowsoffire.apotheosis.adventure.client.AdventureModuleClient;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.placebo.codec.CodecProvider;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import dev.shadowsoffire.placebo.reload.DynamicRegistry;
import net.minecraftforge.fml.loading.FMLEnvironment;

public class AffixRegistry extends DynamicRegistry<Affix> {
  public static final dev.shadowsoffire.apotheosis.adventure.affix.AffixRegistry INSTANCE = new dev.shadowsoffire.apotheosis.adventure.affix.AffixRegistry();
  
  private Multimap<AffixType, DynamicHolder<Affix>> byType = ImmutableMultimap.of();
  
  public AffixRegistry() {
    super(AdventureModule.LOGGER, "affixes", true, true);
  }
  
  protected void beginReload() {
    super.beginReload();
    this.byType = ImmutableMultimap.of();
  }
  
  protected void onReload() {
    super.onReload();
    ImmutableMultimap.Builder<AffixType, DynamicHolder<Affix>> builder = ImmutableMultimap.builder();
    this.registry.values().forEach(a -> builder.put(a.type, holder((CodecProvider)a)));
    this.byType = builder.build();
    Preconditions.checkArgument(Apoth.Affixes.DURABLE.get() instanceof DurableAffix, "Durable Affix not registered!");
    if (!FMLEnvironment.production && FMLEnvironment.dist.isClient())
      AdventureModuleClient.checkAffixLangKeys(); 
    RarityRegistry.INSTANCE.validateLootRules();
  }
  
  protected void registerBuiltinCodecs() {
    registerCodec(Apotheosis.loc("attribute"), AttributeAffix.CODEC);
    registerCodec(Apotheosis.loc("mob_effect"), PotionAffix.CODEC);
    registerCodec(Apotheosis.loc("damage_reduction"), DamageReductionAffix.CODEC);
    registerCodec(Apotheosis.loc("catalyzing"), CatalyzingAffix.CODEC);
    registerCodec(Apotheosis.loc("cleaving"), CleavingAffix.CODEC);
    registerCodec(Apotheosis.loc("enlightened"), EnlightenedAffix.CODEC);
    registerCodec(Apotheosis.loc("executing"), ExecutingAffix.CODEC);
    registerCodec(Apotheosis.loc("festive"), FestiveAffix.CODEC);
    registerCodec(Apotheosis.loc("magical"), MagicalArrowAffix.CODEC);
    registerCodec(Apotheosis.loc("omnetic"), OmneticAffix.CODEC);
    registerCodec(Apotheosis.loc("psychic"), PsychicAffix.CODEC);
    registerCodec(Apotheosis.loc("radial"), RadialAffix.CODEC);
    registerCodec(Apotheosis.loc("retreating"), RetreatingAffix.CODEC);
    registerCodec(Apotheosis.loc("spectral"), SpectralShotAffix.CODEC);
    registerCodec(Apotheosis.loc("telepathic"), TelepathicAffix.CODEC);
    registerCodec(Apotheosis.loc("thunderstruck"), ThunderstruckAffix.CODEC);
    registerCodec(Apotheosis.loc("durable"), DurableAffix.CODEC);
  }
  
  public Multimap<AffixType, DynamicHolder<Affix>> getTypeMap() {
    return this.byType;
  }
}
